/*
   An Implementation of the Edmonds' weighted matching algorithm
   Dec. 1997
   Nakamura Daishin
   daishin@im.uec.ac.jp
*/

/* 	This program is free software; you can redistribute it and/or modify it under the terms of the
 * 	GNU Lesser General Public License as published by the Free Software Foundation; either version
 * 	3.0 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * 	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 	See the GNU Lesser General Public License for more details.
 *
 *  The full text of the license is available at http://www.gnu.org/copyleft/lesser.html
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "datadesc.h"
#include "edmonds.h"

extern int num;
extern int dist();
extern int print_data();

void *mal(size_t siz)
{
  void *p;

  printf("%ld KB\n", (long)siz / 1024);
  p = malloc(siz);  
  if (p == NULL) {
	printf("memory full\n");
	exit(1);
  }
  return(p);
}




int diam;
int upperbound;
double shiftbound;

int diameter(data_type *d, int n, FILE *f)
{
	int i, k, x, y; int h, b; data_type *p, *q;
	h = 0;
	for (p = d + (i = n - 1); i >= 0; --i, --p) {
		for (q = d + (k = i - 1); k >= 0; --k, --q) {
			b = dist(p, q);
			if (b > h) {
			  h = b; x = i; y = k;
			}
		}
	}
	fprintf(f, "%d: ", x); print_data(d + x, f); fprintf(f, "\n");
	fprintf(f, "%d: ", y); print_data(d + y, f); fprintf(f, "\n");
	return(h);
}

/* push leaves */
void push_leaves(vertex_type *z, vertex_type **sp)
{
	vertex_type *b, *p;

	b = z -> base;
	if (b == NULL) {
		push1(z, *sp);
	} else {
		p = b;
		do {
			push_leaves(p, sp);
			p = p -> round_blossom;
		} while (p != b);
	}
}

/* set root blossom of ancestors */
void setroot(vertex_type *x)
{
	vertex_type *b, *p;

	b = x -> base;
	if (b == NULL) {
	} else {
		p = b;
		do {
			p -> root = (x -> root);
			setroot(p);
			p = p -> round_blossom;
		} while (p != b);
	}
}

/* shrink a blossom */
void shrink(vertex_type *e, vertex_type *f,
	vertex_type *b, vertex_type **sp, vertex_type **freeblossom)
{
	vertex_type *p, *q, *r, *s, *v, *w, *z;

	pop1(z, *freeblossom);
	z -> base = b; z -> root = z; z -> parent = NULL;
	z -> prev_tail = b -> prev_tail;
	z -> prev_head = b -> prev_head;
	z -> label = even;
	z -> var = 0;

	for (p = v = e -> root; p != b; p = q) {
		q = p -> prev_head -> root;
		q -> parent = z;
		p -> round_blossom = q;
		if (q -> label == odd) { push_leaves(q, sp); }
	}

	do {
		v -> parent = z;
		w = f -> root;
		r = w -> prev_tail; s = w -> prev_head;
		w -> round_blossom = v;
		w -> prev_tail = f; w -> prev_head = e;
		v -> label = w -> label;
		if (w -> label == odd) { push_leaves(w, sp); }
		e = r; f = s; v = w;
	} while (v != b);

	setroot(z);
}

void changeblossom(vertex_type *, vertex_type *);

/* add an edge to matching */
void addmatch(vertex_type *x, vertex_type *y, vertex_type *e, vertex_type *f)
{
	e -> partner = f; f -> partner = e;
	changeblossom(x, e); changeblossom(y, f);
}

/* change base of blossom from b to x */
void changebase(vertex_type *b, vertex_type *x)
{
	int i; vertex_type *p, *q, *r;

	if (x -> label == even) {
		p = x; r = b; i = 1;
	} else {
		p = b; r = x; i = 0;
	}
	for (; p != r; i = 1 - i, p = q) {
		q = p -> round_blossom;
		if (i) {
			q -> label = even;
		} else {
			addmatch(p, q, p -> prev_tail, p -> prev_head);
			q -> label = odd;
		}
	}
}

/* change blossom */
void changeblossom(vertex_type *u, vertex_type *e)
{
	vertex_type *p, *q;

	if (u -> base == NULL) { /* if u is a vertex, do nothing */
	} else {
		p = e;
		do {
			q = p -> parent;
			changebase(q -> base, p);
			q -> base = p;
			p = q;
		} while (p != u);
	}
}

/* expand a blossom */
void expand(vertex_type *z, vertex_type **freeblossom)
{
	vertex_type *p, *b;

	b = z -> base;
	p = b;
	do {
		p -> root = p; p -> parent = NULL;
		setroot(p);
		p = p -> round_blossom;
	} while (p != b);
	z -> base = NULL; push1(z, *freeblossom);
}

/* expand an odd blossom */
vertex_type *expand_odd(vertex_type *z,
	vertex_type **sp, vertex_type **freeblossom)
{
	int i;
	vertex_type *b, *x, *e, *f, *p, *q, *g, *h;

	b = z -> base;
	e = z -> prev_tail; f = z -> prev_head;
	expand(z, freeblossom);
	x = e -> root;
	if (x -> label == even) {
		for (i = 0, p = x; p != b; p = q, e = g, f = h, i = 1 - i) {
			q = p -> round_blossom;
			g = p -> prev_head; h = p -> prev_tail;
			p -> prev_tail = e; p -> prev_head = f;
			if (i) {
				p -> label = even;
				push_leaves(p, sp);
			} else {
				p -> label = odd;
			}
		}
		do {
			p -> label = unlabeled;
			p = p -> round_blossom;
		} while (p != x);
		b -> prev_tail = e; b -> prev_head = f;
		b -> label = odd;
	} else {
		i = 0; p = b;
		do {
			q = p -> round_blossom;
			if (i) {
				p -> label = even;
				push_leaves(p, sp);
			} else {
				p -> label = odd;
			}
			p = q; i = 1 - i;
		} while (p != x);
		do {
			p -> label = unlabeled;
			p = p -> round_blossom;
		} while (p != b);
		x -> prev_tail = e; x -> prev_head = f;
		x -> label = odd;
	}
	return(b);
}

/* expand all blossoms with zero dual variable */
void expand_zero(vertex_type *v, int n, vertex_type **freeblossom)
{
	int i; vertex_type *p;

	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		while ((p -> root != p) && (zerop(p -> root -> var)))
			expand(p -> root, freeblossom);
	}
}

/* augmenting(half) */
void aughalf(vertex_type *x)
{
	vertex_type *p, *q, *e, *f;

	for (p = x; (e = p -> prev_tail) != NULL; p = q) {
		f = p -> prev_head;
		q = f -> root;
		if (p -> label == even) {
		} else {
			addmatch(p, q, e, f);
		}
	}
}

/* augmenting */
void augment(vertex_type *x, vertex_type *y, vertex_type *e, vertex_type *f)
{
	aughalf(x); aughalf(y);
	if (x -> label == even) addmatch(x, y, e, f);
}

/* find nearest common ancestor of v and w */
vertex_type *findcommon(vertex_type *v, vertex_type *w)
{
	int i, k; vertex_type *p, *q, *b;

	/* search */
	i = (v -> label == even); k = (w -> label == even);
	v -> label = w -> label = unlabeled;
	for (p = v, q = w, b = NULL;
		(p -> prev_head != NULL) || (q -> prev_head != NULL);) {
		if (p -> prev_head != NULL) {
			p = p -> prev_head -> root;
			if (p -> label == unlabeled) {
				b = p; break; /* found ! */
			}
			p -> label = unlabeled;
		}
		if (q -> prev_head != NULL) {
			q = q -> prev_head -> root;
			if (q -> label == unlabeled) {
				b = q; break; /* found ! */
			}
			q -> label = unlabeled;
		}
	}
	/* recover labels */
	for (p = v; p -> label == unlabeled; i = !i) {
		if (i) p -> label = even; else p -> label = odd;
		if (p -> prev_head != NULL) {
			p = p -> prev_head -> root;
		}
	}
	for (p = w; p -> label == unlabeled; k = !k) {
		if (k) p -> label = even; else p -> label = odd;
		if (p -> prev_head != NULL) {
			p = p -> prev_head -> root;
		}
	}
	return (b);
}

void new_odd(vertex_type *e, vertex_type *f, vertex_type *y, vertex_type **sp)
{
	vertex_type *p, *q, *z;

	y -> prev_tail = f; y -> prev_head = e;
	y -> label = odd;
	for(p = y; p -> base != NULL; p = p -> base);
	q = p -> partner;
	if (q != NULL) {
		z = q -> root;
		if (z -> label == unlabeled) {
			z -> label = even;
			z -> prev_tail = q; z -> prev_head = p;
			push_leaves(z, sp);
		}
	}
}

/* Edmonds' blossom algorithm for cardinarity matching */
int cardinarity(vertex_type *v/* array of vertices */
	,int n/* # of vertices */
	,vertex_type *u/* array for blossoms (size n / 2) */
	,int c) /* starting matching size */
{
	int i, m;
	vertex_type *b, *p, *q, *r, *x, *y, *z, *sp, *freeblossom;

	/* start */
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> root = p; p -> parent = NULL; p -> base = NULL;
	}
	for (freeblossom = NULL, p = u + (i = n / 2 - 1); i >= 0; --i, --p) {
		p -> base = NULL; push1(p, freeblossom);
	}
	m = n / 2; /* maximum cardinarity */

	/* clear */
l1:
	if ((c & 63) == 0) printf("%d\n", c);
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> root -> label = unlabeled;
	}
	if (c == m) return(c);
	for (sp = NULL, p = v + (i = n - 1); i >= 0; --i, --p) {
		if (p -> partner == NULL) {
			x = p -> root;
			x -> label = even;
			x -> prev_tail = x -> prev_head = NULL;
			push_leaves(x, &sp);
		}
	}

	/* growing alternating tree */
	while (sp != NULL) {
		pop1(p, sp);
		LOOP(p, q)
			x = p -> root; y = q -> root;
			if (x == y) { /* self loop */
			} else if (!ADJ(p, q)) { /* not adjacent */
			} else if (p -> partner == q) { /* matched with q */
			} else if (y -> label == unlabeled) {
				while (y != q) {
					r = b = y -> base;						
					do {
						r -> label = unlabeled;
						r = r -> round_blossom;
					} while (r != b);
					expand(y, &freeblossom);
					y = q -> root;
				}
				y -> prev_tail = q; y -> prev_head = p;
				y -> label = odd;
				r = q -> partner;
				if (r != NULL) {
					z = r -> root;
					if (z -> label == unlabeled) {
						z -> label = even;
						z -> prev_tail = r; z -> prev_head = q;
						push_leaves(z, &sp);
					}
				}
			} else if (y -> label == even) {
				b = findcommon(x, y);
				if (b == NULL) {
					augment(x, y, p, q);
					++c;
					goto l1;
				} else {
					shrink(p, q, b, &sp, &freeblossom);
				}
			}
		END
	}
	return(c);
}

/* Edmonds' blossom algorithm for weighted matching */
int weighted(vertex_type *v/* array of vertices */
	,int n/* # of vertices */
	,vertex_type *u)/* array for blossoms (size n / 2) */
{
	int i, m, c; double w, vm, eu, ee, od;
	vertex_type *b, *p, *q, *x, *y, *sp, *freeblossom;

	/* start */
	vm = -infty;
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		LOOP(p, q)
			if (p < q) {
				w = WEIGHT(p, q);
				if (greaterp(w, vm)) vm = w;
			}
		END
	}
	vm *= 0.5;
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> root = p; p -> parent = NULL; p -> base = NULL;
		p -> partner = NULL;
		p -> var = vm;
	}
	for (freeblossom = NULL, p = u + (i = n / 2 - 1); i >= 0; --i, --p) {
		p -> base = NULL; push1(p, freeblossom);
	}
	m = n / 2; /* maximum cardinarity */
	c = 0;

	/* clear */
l1:
	if ((c & 63) == 0) printf("%d\n", c);
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> root -> label = unlabeled;
	}
	/*if (c == m) return(c);*/
	for (sp = NULL, p = v + (i = n - 1); i >= 0; --i, --p) {
		if (p -> partner == NULL) {
			x = p -> root;
			x -> label = even;
			x -> prev_tail = x -> prev_head = NULL;
			push_leaves(x, &sp);
		}
	}

	/* growing alternating tree */
l3:
	while (sp != NULL) {
		pop1(p, sp);
		LOOP(p, q)
			x = p -> root; y = q -> root;
			if (x == y) { /* self loop */
			} else if (!ADJ(p, q)) { /* not adjacent */
			} else if (greaterp((p -> var) + (q -> var), WEIGHT(p, q))) {
			} else if (p -> partner == q) { /* matched with q */
			} else if (y -> label == unlabeled) {
				new_odd(p, q, y, &sp);
			} else if (y -> label == even) {
				b = findcommon(x, y);
				if (b == NULL) {
					augment(x, y, p, q);
					++c;
					expand_zero(v, n, &freeblossom);
					goto l1;
				} else {
					shrink(p, q, b, &sp, &freeblossom);
				}
			}
		END
	}

	/* dual change */
	for (eu = ee = od = infty, p = v + (i = n - 1); i >= 0; --i, --p) {
		x = p -> root;
		if (x -> label == even) {
			LOOP(p, q)
				y = q -> root;
				if (x == y) { /* self loop */
				} else if (!ADJ(p, q)) { /* not adjacent */
				} else if (y -> label == unlabeled) {
					w = (p -> var) + (q -> var) - (WEIGHT(p, q));
					if (greaterp(eu, w)) eu = w;
				} else if (y -> label == even) {	shiftbound = 0;

					w = (p -> var) + (q -> var) - (WEIGHT(p, q));
					if (greaterp(ee, w)) ee = w;
				}
			END
		} else if (x != p && x -> label == odd) {
			if (greaterp(od, x -> var)) od = x -> var;
		}
	}
	ee *= 0.5;
	od *= 0.5;
	w = vm;
	if (greaterp(w, eu)) w = eu;
	if (greaterp(w, ee)) w = ee;
	if (greaterp(w, od)) w = od;

	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		x = p -> root;
		if (x -> label == even) { p -> var -= w;
		} else if (x -> label == odd) { p -> var += w; }
	}
	vm -= w;
	w *= 2;
	for (p = v + (n + (i = n / 2 - 1)); i >= 0; --i, --p) {
		if (p -> base == NULL) {
		} else if (p -> root != p) {/* inner blossom */
		} else if (p -> label == even) { p -> var += w;
		} else if (p -> label == odd) { p -> var -= w; }
	}
	if (zerop(vm)) return(c); /* maximum matching was found */

	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		if ((x = p -> root) != p
			&& (x -> label == odd) && zerop(x -> var)) {
			expand_odd(x, &sp, &freeblossom);
		}
	}
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		LOOP(p, q)
			x = p -> root; y = q -> root;
			if (x == y) { /* self loop */
			} else if (!ADJ(p, q)) { /* not adjacent */
			} else if (p -> partner == q) {
			} else if (x -> label != even) {
			} else if (greaterp((p -> var) + (q -> var),
				WEIGHT(p, q))) {
			} else if (y -> label == unlabeled) {
				new_odd(p, q, y, &sp);
			} else if (y -> label == even) {
				b = findcommon(x, y);
				if (b == NULL) {
					augment(x, y, p, q);
					++c;
					expand_zero(v, n, &freeblossom);
					goto l1;
				} else {
					shrink(p, q, b, &sp, &freeblossom);
				}
			}
		END
	}
	goto l3;
}

int match_check(vertex_type *v, int n)
{
	int i, c; vertex_type *p, *q;

	c = 0;
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		q = p -> partner;
		if (p == q) {
			printf("(%d, %d) is a loop\n", (int)(p - v), (int)(p - v));
			exit(1);
		} else if (q != NULL) {
			if (q -> partner != p) {
				printf("(%d, %d) ?\n", (int)(p - v), (int)(q - v));
				q = q -> partner;
				if (q == NULL) printf("NULL\n");
				else printf("%d\n", (int)(q - v));
				exit(1);
			}
			++c;
		}
	}
	printf("matching checked: size=%d\n", c / 2);
	return(c / 2);
}

void cover_check(vertex_type *v, int n, vertex_type *u)
{
	int i, k, s;
	vertex_type *p, *q, *e, *f;

	for (p = u + (i = n / 2 - 1); i >= 0; --i, --p) {
		if (p -> base == NULL) {
		} else if (p -> root -> label == odd) {
			printf("odd blossom\n");
			exit(1);
		} else {
			q = p -> base; k = 0; s = 0;
			do {
				e = q -> prev_tail; f = q -> prev_head;
				if (e -> partner == f) {
					if (!k) {
						printf("irregal blossom\n");
						exit(1);
					}
				} else {
					if (k) {
						printf("irregal blossom\n");
						exit(1);
					}
				}
				q = q -> round_blossom; k = 1 - k; ++s;
			} while (q != p -> base);
			if ((s < 3) || (s % 2 == 0)) {
				printf("irregal blossom: size %d\n", s);
				exit(1);
			}
		}
	}
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		LOOP(p, q)
			if (p -> root -> label == unlabeled) {
				if (p -> partner == NULL) {
					printf("unlabeled vertex %d is not matched\n", (int)(p - v));
					exit(1);
				}
			}
			if (p < q && ADJ(p, q)) {
				if ((p -> root == q -> root)
					|| (p == p -> root) && (p -> label == odd)
					|| (q == q -> root) && (q -> label == odd)
					|| (p -> root -> label == unlabeled) && (q -> root -> label == unlabeled)
				) {
				} else {
					printf("cover is invalid\n");
					exit(1);
				}
			}
		END
	}
	printf("cover checked, OK!\n");
}

vertex_type *commonblossom(vertex_type *v, vertex_type *w)
{
	vertex_type *p, *q, *r, *b;

	r = v -> root;
	if (w -> root != r) return(NULL);

	v -> root = w -> root = NULL;
	for (p = v, q = w, b = NULL; (p -> parent != NULL) || (q -> parent != NULL);) {
		if (p -> parent != NULL) {
			p = p -> parent;
			if (p -> root == NULL) {
				b = p; break; /* found ! */
			}
			p -> root = NULL;
		}
		if (q -> parent != NULL) {
			q = q -> parent;
			if (q -> root == NULL) {
				b = q; break; /* found ! */
			}
			q -> root = NULL;
		}
	}
	/* recover */
	for (p = v; (p != NULL) && (p -> root == NULL); p = p -> parent) p -> root = r;
	for (p = w; (p != NULL) && (p -> root == NULL); p = p -> parent) p -> root = r;
	return (b);
}

void dual_check(vertex_type *v, int n, vertex_type *u)
{
	int i, k, c; double s, w;
	vertex_type *p, *q, *r, *e, *f;

	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		if (greaterp(0, p -> var)) {
			printf("dual variable on vertex %d is minus\n", i);
			exit(1);
		} else if (greaterp(p -> var, 0) && (p -> partner == NULL)) {
			printf("dual variable of unmatched vertex %d is plus(%f)\n",
			    i, p -> var);
			exit(1);
		}
	}
	for (p = u + (i = n / 2 - 1); i >= 0; --i, --p) {
		if (p -> base == NULL) {
		} else if (greaterp(0, p -> var)) {
			printf("dual variable of an blossom is minus\n");
			exit(1);
		} else if (greaterp(p -> var, 0)) {
			q = p -> base; k = 0; c = 0;
			do {
				e = q -> prev_tail; f = q -> prev_head;
				if (e -> partner == f) {
					if (!k) {
						printf("irregal blossom\n");
						exit(1);
					}
				} else {
					if (k) {
						printf("irregal blossom\n");
						exit(1);
					}
				}
				q = q -> round_blossom; k = 1 - k; ++c;
			} while (q != p -> base);
			if ((c < 3) || (c % 2 == 0)) {
				printf("irregal blossom: size %d\n", c);
				exit(1);
			}
		}
	}
	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		LOOP(p, q)
			if (p < q && ADJ(p, q)) {
				w = WEIGHT(p, q);
				s = (p -> var) + (q -> var);
				for (r = commonblossom(p, q); r != NULL; r = r -> parent)
					s += (r -> var);
				if (greaterp(w, s)) {
					printf("dual infeasible %f < %f on (%d, %d)\n",
						s, w, (int)(p - v), (int)(q - v));
					exit(1);
				} else if (p -> partner == q && greaterp(s, w)){
					printf("dual inequality %f > %f holds on matching (%d, %d)\n",
						s, w, (int)(p - v), (int)(q - v));
					exit(1);
				}				
			}
		END
	}
	printf("dual checked, OK!\n");
}

int get_matching(vertex_type *v, int n, int *m)
{
	int i, c; vertex_type *p, *q;

	for (c = 0, p = v + (i = n - 1); i >= 0; --i, --p) {
		q = p -> partner;
		if (q != NULL && p < q) {
			*(m++) = (int)(p - v);
			*(m++) = (int)(q - v);
			++c;
		}
	}
	return(c);
}

int set_matching(vertex_type *v, int n, int *m, int c, int ub)
{
	int i, k; vertex_type *p, *q;

	for (p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> partner = NULL;
	}
	for (k = 0, i = 0; i < c; ++i) {
		p = v + *(m++); q = v + *(m++);
		if (dist(p -> item, q -> item) <= ub) {
			p -> partner = q; q -> partner = p;
			++k;
		}
	}
	return(k);
}

void write_matching(vertex_type *v, int n, int *m, int c, FILE *f)
{
	int i, p, q;

	fprintf(f, "# of matching edges: %d\n", c);
	for (i = 0; i < c; ++i) {
		p = *(m++);
		q = *(m++);
		fprintf(f, "(%d,%d) %d ", p, q,
		    dist((v + p) -> item, (v + q) -> item));
		print_data((v + p) -> item, f);
		print_data((v + q) -> item, f);
		fprintf(f, "\n");
	}
	fprintf(f, "# of unmatched vertices: %d\n", n - 2 * c);
	for (i = 0; i < n; ++i) {
		if ((v + i) -> partner == NULL) {
			fprintf(f, "%d ", i);
		    print_data((v + i) -> item, f);
			fprintf(f, "\n");
		}
	}
	fprintf(f, "\n");
}

void sum_matching(vertex_type *v, int *m, int c, FILE *f)
{
	int i, d, h, l, p, q, t[512]; double s;

	h = -32767; l = 32767; s = 0;
	for (i = 0; i < 512; ++i) t[i] = 0;
	for (i = 0; i < c; ++i) {
		p = *(m++);
		q = *(m++);
		d = dist((v + p) -> item, (v + q) -> item);
		if (d > h) h = d;
		if (d < l) l = d;
		if (0 <= d && d < 512) ++t[d];
		s += d;
	}
	fprintf(f, "total: %f\n", s);
	fprintf(f, "max: %d\n", h);
	fprintf(f, "min: %d\n", l);
	fprintf(f, "mean: %f / %d = %f\n", s, c, s / c);
	fprintf(f, "distribution:\n");
	for (i = l; i <= h; ++i) {
	  if (0 <= i && i < 512) {
		fprintf(f, "%d:%d ", i, t[i]);
	  }
	}
	fprintf(f, "\n");
}

void count_degree(data_type *d, int n,
	int (*f)(data_type *, data_type *), int upperbound)
{
	int i, k, c, l, h; int b; double s;
	data_type *p, *q;

	s = 0; l = 32767; h = -32767;
	for (p = d + (i = n - 1); i >= 0; --i, --p) {
		c = 0;
		for (q = d + (k = n - 1); k >= 0; --k, --q) {
			if (i != k) {
				b = f(p, q);
				if (b <= upperbound) ++c;
			}
		}
		s += c;
		if (l > c) l = c;
		if (h < c) h = c;
	}
	printf("bound:%d\n", upperbound);
	printf("edge:%f\n", s / 2);
	printf("min degree:%d\n", l);
	printf("max degree:%d\n", h);
	printf("ave degree: %f / %d = %f\n", s, n, s / n);
}

typedef struct {
	int key;
	int no;
} wn_type;

#define LT(x,y) (h[x].key < h[y].key)
#define EX(x,y) t = h[x]; h[x] = h[y]; h[y] = t
void heapify(wn_type *h, int n, int x)
{
	int y; wn_type t;

	while((y = 2 * x + 1) < n) {
		if (y + 1 < n)	{
			if (LT(y + 1, y)) ++y;
		}
		if (!LT(y, x)) break;
		EX(x, y);
		x = y;
	}
}
void heap_sort(wn_type *h, int n, int c)
{
	int i; wn_type t;

	for(i = (n - 1) / 2; i >= 0; --i) heapify(h, n, i);
	for(i = 1; i <= c; ++i) {
		EX(0, n - i);
		heapify(h, n - i, 0);
	}
}
#undef LT
#undef EX

void make_adj(vertex_type *v, int n, int c, adj_type *a)
{
	int i, k; double w; vertex_type *p, *q; wn_type *h, *x;

	h = MALLOC(wn_type, n - 1);
	if (h == NULL) { printf("Memory full!\n"); exit(1); }

	for(p = v + (i = n - 1); i >= 0; --i, --p) {
		p -> adj_list = NULL;
	}

	for(p = v + (i = n - 1); i >= 0; --i, --p) {
		for(x = h, q = v + (k = n - 1); k >= 0; --k, --q) {
			if (i != k) {
				x -> key = (dist(p -> item, q -> item));	
				x -> no = k;
				++x;
			}
		}
		heap_sort(h, n - 1, c);
		for(x = h + ((n - 1) - 1), k = 0; k < c; ++k, --x) {
			q = v + (x -> no);
			w = SHIFT(x -> key);
			a -> head_vertex = q;
			a -> weight = w;
			a -> next = p -> adj_list;
			p -> adj_list = a;
			++a;
			a -> head_vertex = p;
			a -> weight = w;
			a -> next = q -> adj_list;
			q -> adj_list = a;
			++a;
		}
	}
	FREE(h);
}


