/*
   An Implementation of the Edmonds' weighted matching algorithm
   Dec. 1997
   Nakamura Daishin
   daishin@im.uec.ac.jp
*/

/* 	This program is free software; you can redistribute it and/or modify it under the terms of the
 * 	GNU Lesser General Public License as published by the Free Software Foundation; either version
 * 	3.0 of the License, or (at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * 	without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 	See the GNU Lesser General Public License for more details.
 *
 *  The full text of the license is available at http://www.gnu.org/copyleft/lesser.html
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "datadesc.h"
#include "edmonds.h"

int num;

extern int diam;
extern double shiftbound;

extern int weighted();
extern void match_check();
extern void dual_check();
extern int get_matching();
extern void sum_matching();
extern void write_matching();
extern int diameter();
extern void *mal();
extern int read_file();
extern void make_adj();



int main(void)
{
	int i;
	int c2;
	int *m1, *m2;
	vertex_type *v;
	data_type *data;
	adj_type *a;
	int deg;
	FILE *f;
	char p[256];
	long t1, t2;

	printf("number of vertices; n = ");
	scanf("%d",&num); 

	printf("nearest k vertices(1 to %d); k = ", num - 1);
	scanf("%d",&deg); 

	if (deg >= num || deg < 1) {
	   printf("n and k are incorrect\n"); exit(1);
	}

	printf("data file = ");
	scanf("%s", p);

	data = MALLOC(data_type, num);
	if (read_file(p, num, data) != num) {
	    printf("can\'t read data file correctly\n");
		exit(1);
	}

	f = fopen("a.log", "w");

	v = MALLOC(vertex_type, (((long)num) * 3) / 2);
	for (i = 0; i < num; ++i) v[i].item = (data + i);

	m1 = MALLOC(int, 2 * ((long)num));
	m2 = m1 + num;

	a = MALLOC(adj_type, 2 * ((long)num) * deg);

	printf("vertices:%d\n", num);
	fprintf(f, "vertices:%d\n", num);
	printf("nearest %d vertices\n", deg);
	fprintf(f, "nearest %d vertices\n", deg);

	diam = diameter(data, num, f);

	printf("diameter: %d\n", diam);
	fprintf(f, "diameter: %d\n", diam);

	time(&t1);
	make_adj(v, num, deg, a);
	time(&t2);
	printf("adjacency list (nearest %d vertices): %ld sec\n",
		   deg, (long)(t2 - t1));
	fprintf(f, "adjacency list (nearest %d vertices): %ld sec\n",
		   deg, (long)(t2 - t1));

	printf("*** weighted ***\n");
	fprintf(f, "*** weighted ***\n");
	shiftbound = 0;
	printf("bound:%d\n", UPPERBOUND);
	fprintf(f, "bound:%d\n", UPPERBOUND);

	time(&t1);
	weighted(v, num, v + num);
	time(&t2);
	printf("maximum weight matching: %ld sec\n", (long)(t2 - t1));
	fprintf(f, "maximum weight matching: %ld sec\n", (long)(t2 - t1));
	match_check(v, num);
	dual_check(v, num, v + num);
	c2 = get_matching(v, num, m2);
	sum_matching(v, m2, c2, stdout);
	sum_matching(v, m2, c2, f);
	write_matching(v, num, m2, c2, f);

	FREE(a);

	FREE(m1);
	FREE(v);
	FREE(data);
	fclose(f);
	return(0);
}
